@@ -25,9 +25,11 @@ from azure.mgmt.containerinstance.models import (
 import json
 import os
 from azure.common.credentials import ServicePrincipalCredentials
+from datetime import datetime
 
 from taz.auth import UserAssignedIdentity, ClientSecretAuthentication, GenericClient
 from taz.exception import ContainerAlreadyExistsException
+from taz.blob import Blob
 
 
 class SimpleContainerGroup:
@@ -118,6 +120,8 @@ class SimpleContainerGroup:
         self.protected = True
         self.env_vars = []
         self.identity_name = identity_name
+        self.container_name = "container-001"
+        self.last_finish_time = None
 
         if env_vars:
             self.set_env_vars(env_vars)
@@ -206,7 +210,7 @@ class SimpleContainerGroup:
         self.identity_name = identity_name
         self.identity = UserAssignedIdentity(
             resource_group, self.identity_name, subscription_id=self.subscription_id
-        )
+        ).get_container_group_identity()
 
     def set_command(self, command):
         """
@@ -234,7 +238,7 @@ class SimpleContainerGroup:
         )
 
         container = Container(
-            name="container-001",
+            name=self.container_name,
             image="{0}.azurecr.io/{1}:latest".format(
                 self.image.container_registry.name, self.image.name
             ),
@@ -309,6 +313,48 @@ class SimpleContainerGroup:
 
         return instances_logs
 
+    def save_logs(
+        self,
+        container,
+        sas_url=None,
+        sas_key=None,
+        storage_account_name=None,
+        storage_account=None,
+    ):
+
+        datas_group_logs = self.__str__()
+        datas_container_logs = self.list_logs()
+
+        if self.last_finish_time:
+            name_group_log = "taz.aci.group.{}.{}.log".format(
+                self.last_finish_time, self.name
+            )
+            name_container_log = "taz.aci.container.{}.{}.{}.log".format(
+                self.last_finish_time, self.name, self.container_name,
+            )
+            blob_group_logs = Blob(
+                container,
+                name_group_log,
+                sas_url=sas_url,
+                sas_key=sas_key,
+                storage_account_name=storage_account_name,
+                storage_account=storage_account,
+            )
+            blob_container_logs = Blob(
+                container,
+                name_container_log,
+                sas_url=sas_url,
+                sas_key=sas_key,
+                storage_account_name=storage_account_name,
+                storage_account=storage_account,
+            )
+            blob_group_logs.write(datas_group_logs)
+            blob_container_logs.write(datas_container_logs)
+
+            return name_group_log, name_container_log
+        else:
+            return None
+
     def __str__(self):
         """
         prints container group as json string
@@ -343,6 +389,7 @@ class SimpleContainerGroup:
                 state = instance.instance_view.current_state.state
                 start_time = str(instance.instance_view.current_state.start_time)
                 finish_time = str(instance.instance_view.current_state.finish_time)
+                self.last_finish_time = finish_time
                 exit_code = instance.instance_view.current_state.exit_code
                 detail_status = instance.instance_view.current_state.detail_status
                 memory = instance.resources.requests.memory_in_gb
